using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;

namespace Clouds
{

    /// <summary>
    /// This is the main type for your game
    /// </summary>
    public class CloudGame : Microsoft.Xna.Framework.Game
    {
        GraphicsDeviceManager graphics;
        SpriteBatch spriteBatch;

        // Game World

        Random rand = new Random();

        int noOfSprites = 10;
        List<ISprite> gameSprites = new List<ISprite>();

        public interface ISprite
        {
            void Draw(CloudGame game);
            void Update(CloudGame game);
        }

        public class Cloud : CloudGame.ISprite
        {
            public Texture2D CloudTexture;
            public Vector2 CloudPosition;
            public Vector2 CloudSpeed;

            virtual public void Draw(CloudGame game)
            {
                game.spriteBatch.Draw(CloudTexture, CloudPosition, Color.White);
            }

            virtual public void Update(CloudGame game)
            {
                CloudPosition += CloudSpeed;

                if (CloudPosition.X > game.GraphicsDevice.Viewport.Width)
                    CloudPosition.X = -CloudTexture.Width;

            }

            public Cloud(Texture2D inTexture, Vector2 inPosition, Vector2 inSpeed)
            {
                CloudTexture = inTexture;
                CloudPosition = inPosition;
                CloudSpeed = inSpeed;
            }
        }


        public CloudGame()
        {
            graphics = new GraphicsDeviceManager(this);
            Content.RootDirectory = "Content";
        }

        /// <summary>
        /// Allows the game to perform any initialization it needs to before starting to run.
        /// This is where it can query for any required services and load any non-graphic
        /// related content.  Calling base.Initialize will enumerate through any components
        /// and initialize them as well.
        /// </summary>
        protected override void Initialize()
        {
            // TODO: Add your initialization logic here

            base.Initialize();
        }

        /// <summary>
        /// LoadContent will be called once per game and is the place to load
        /// all of your content.
        /// </summary>
        protected override void LoadContent()
        {
            // Create a new SpriteBatch, which can be used to draw textures.
            spriteBatch = new SpriteBatch(GraphicsDevice);

            Texture2D cloudTexture = Content.Load<Texture2D>("Cloud");
            Rectangle cloudRectangle = new Rectangle(0, 0, cloudTexture.Width, cloudTexture.Height);

            for (int i = 0; i < noOfSprites; i++)
            {
                Vector2 position =
                    new Vector2(rand.Next(GraphicsDevice.Viewport.Width),
                                rand.Next(GraphicsDevice.Viewport.Height));

                //Vector2 speed = new Vector2(rand.Next(0, 100) / 100f, 0); 

                // To get a nice parallax scrolling effect we can make the 
                // top level clouds move more quickly
                Vector2 speed = new Vector2(i / 6f, 0);

                Cloud c = new Cloud(cloudTexture, position, speed);

                gameSprites.Add(c);
            }
        }

        /// <summary>
        /// UnloadContent will be called once per game and is the place to unload
        /// all content.
        /// </summary>
        protected override void UnloadContent()
        {
            // TODO: Unload any non ContentManager content here
        }

        public KeyboardState CurrentKeyboardState;
        
        /// <summary>
        /// Allows the game to run logic such as updating the world,
        /// checking for collisions, gathering input, and playing audio.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Update(GameTime gameTime)
        {
            // Allows the game to exit
            if (GamePad.GetState(PlayerIndex.One).Buttons.Back == ButtonState.Pressed)
                this.Exit();

            CurrentKeyboardState = Keyboard.GetState();
            foreach (ISprite sprite in gameSprites)
                sprite.Update(this);

            base.Update(gameTime);
        }

        /// <summary>
        /// This is called when the game should draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Draw(GameTime gameTime)
        {
            GraphicsDevice.Clear(Color.CornflowerBlue);
            spriteBatch.Begin();

            foreach (ISprite sprite in gameSprites)
                sprite.Draw(this);

            spriteBatch.End();
            base.Draw(gameTime);
        }
    }
}
